#include <SPI.h>
#include <Ethernet.h>
#include <MFRC522.h>
#include <LiquidCrystal.h>
#include <DS3231.h>


/* START RFID */
#define RST_PIN 48
#define SS_PIN  53

String read_rfid;
String userName;
String ok_rfid_1 = "268743"; //adicionar os "UID" das tags RFID(cartões, porta-chaves, etc).
String ok_rfid_2 = "364a2311"; //podemos adicionar quantos quisermos
String ok_rfid_3 = "b6e22011";
String ok_rfid_4 = "86f61c11";
String ok_rfid_5 = "013dda1";
String ok_rfid_6 = "86951911";
String ok_rfid_7 = "2692f11";
String ok_rfid_8 = "d0aedda1";
String ok_rfid_9 = "16c81a11";
String ok_rfid_10 = "36172a11";
String ok_rfid_11 = "6491311";

MFRC522 mfrc522(SS_PIN, RST_PIN);  //instanciar o MFRC522
/*END RFID*/


/* Inicializar o LCD */
LiquidCrystal lcd(22, 23, 26, 27, 28, 29);
/* LCD */

/* START Ethernet */
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 0, 177); //coloque o Ip para o Arduino, cuidado não usar o ponto (".") e sim a virgula (",").
char server[] = "www.o_vosso_site.pt"; //Importante, se usarem o XAMP localmente teem que colocar o IP do vosso computador, se for online coloquem o vosso site "www.exemplo.pt"
EthernetClient client;
/*END Ethernet */

/* Server authentication */
String auth = "bf98c3bf08764d1583c29b2863c875e7"; //variavel para autenticar, pode ser gerada ou então uma palavra qualquer vossa, e tem que fazer match com a que esta no db.ini.
boolean success = false;

// Iniciar o DS3231
DS3231  rtc(SDA, SCL);
String hora, data;
String separador = "-"; //Separador para separar a hora da data na BD

//buzzer
const int buzzer = 49;

void setup() {
  //Start Serial
  Serial.begin(9600);

  rtc.begin();

  // Inicializa SPI bus
  SPI.begin();
  mfrc522.PCD_Init();// Inicializa MFRC522

  //Inicializar o LCD.
  lcd.begin(16, 2);
  lcd.clear();
  lcd.print("A ligar...");
  //Serial.println("A ligar..."); //Debug opcional

  // Start conexão Ethernet
  Ethernet.begin(mac, ip);
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // tentar configurar usando um IP address ao inves do DHCP.
    Ethernet.begin(mac, ip);
  }
  //Dar 1 segundo para a placa Ethernet Inicializar.
  pinMode(buzzer, OUTPUT);
  delay(1000);
}

void loop() {
  String timestamp="";
  timestamp = getTime();

  //Rfid
  if (!mfrc522.PICC_IsNewCardPresent())
    return;
  if (!mfrc522.PICC_ReadCardSerial())
    return;

  //corre a função implementada para ler a tag.
  dump_byte_array(mfrc522.uid.uidByte, mfrc522.uid.size);

  Serial.println(read_rfid); //Debug para ver o ID da tag (opcional)
  userName = getUserFromTagId(read_rfid);

  if (userName.length() > 0 && userName != "") {
    if (userName == "notRecognized") {
      makeErrorBip();
      printToLCD("Nao", "Autorizado");
    } else {
      makeBip();
      success = sendToServer(userName, timestamp);
      // se enviou com sucesso
      if (success) {
        printToLCD("Bem vindo", userName);
      } else {
        Serial.println("--> Ligacao Falhada\n");
        printToLCD("Ligacao", "Falhada");
      }
    }
  }
}

void makeBip() {
  tone(buzzer, 4000); // Send 4KHz sound signal...
  delay(300);        // ...for 300 miliSec
  noTone(buzzer);     // Stop sound...
}

void makeErrorBip() {
  for (int i = 0; i < 5; i++) {
    tone(buzzer, 1000); // Send 4KHz sound signal...
    delay(150);        // ...for 150 miliSec
    noTone(buzzer);     // Stop sound...
  }
}

String getTime() {
  hora = rtc.getTimeStr();
  data = rtc.getDateStr();
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print(data);
  lcd.setCursor(4, 1);
  lcd.print(hora);
  delay(1000); 
  return data + separador + hora;
}

//Retorna o nome da pessoa de cada Tag
String getUserFromTagId(String rfid) {
  if (read_rfid == ok_rfid_1) {
    return "User 1";
  } else if (read_rfid == ok_rfid_2) {
    return "User 2";
  } else if (read_rfid == ok_rfid_3) {
    return "User 3";
  } else if (read_rfid == ok_rfid_4) {
    return "User 4";
  } else if (read_rfid == ok_rfid_5) {
    return "User 5";
  } else if (read_rfid == ok_rfid_6) {
    return "User 6";
  } else {
    return "notRecognized"; //Tag não reconhecida, é necessario declara-la no inicio
  }
}


/*
   *  Função que ajuda a escrever no monitor serie o valor do UID das tags. 
   *  NOTA: Não é importante saber como funciona se estás num nivel mais basico.
   */
void dump_byte_array(byte *buffer, byte bufferSize) {
  read_rfid = "";
  for (byte i = 0; i < bufferSize; i++) {
    read_rfid = read_rfid + String(buffer[i], HEX);
  }
}

boolean sendToServer(String userName, String timestamp) {

  if (client.connect(server, 80)) {
    //Serial.println("Ligacao com Sucesso!"); //Debug (opcional)
    
    client.print("GET /scheduleRegister.php?"); // Alterar para o nome do nosso script php na raíz do nosso servidor
    client.print("username="); // variavel com o nosso valor
    client.print(userName); // valor que queremos passar para inserir na BD.
    client.print("&time=");
    client.print(timestamp);
    client.print("&auth=");
    client.print(auth);
    client.println(" HTTP/1.1"); // Parte do HTTP Request
    client.println("Host: www.rascunhofiel.pt"); // IMPORTANT: O vosso site, se usarem XAMP precisam saber o IP do vosso computador.
    client.println("Connection: close"); // Parte do HTTP Request para fechar o pedido.
    client.println(); // Empty line
    client.println(); // Empty line
    client.stop();    // Fecha a ligacao

    return true;
  }
  // Se o arduino nao se conseguir ligar ao servidor retorna false.
  return false;
}

void printToLCD(String firstString, String secondString) {
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print(firstString);
  lcd.setCursor(3, 1);
  lcd.print(secondString);
  delay(2000);
  lcd.clear();
}
